package net.wanji.utc.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hikvision.artemis.sdk.ArtemisHttpUtil;
import com.hikvision.artemis.sdk.config.ArtemisConfig;
import lombok.extern.slf4j.Slf4j;
import net.wanji.utc.common.Result;
import net.wanji.utc.common.baseentity.BaseCrossInfo;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.common.exception.ControlException;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.service.ControlCommandService;
import net.wanji.utc.service.HkGetSignalMethodService;
import net.wanji.utc.vo.PhaseLock;
import net.wanji.utc.vo.signal.SignalLightStateVo;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static net.wanji.utc.common.constant.Constants.*;

/**
 * @author wuxiaokai
 * @date 2022/11/21 9:51:47
 */
@Slf4j
@Service("hkControlCommandService")
public class HKControlCommandServiceImpl implements ControlCommandService {

    @Resource
    private HkGetSignalMethodService hkGetSignalMethodService;

    @Resource
    private ArtemisConfig artemisConfig;

    @Override
    public <T> Result<T> schemeSend() {
        return null;
    }

    @Override
    public <T> Result<T> planSend() {
        return null;
    }

    @Override
    public <T> Result<T> scheduleSend() {
        return null;
    }

    @Override
    public <T> Result<T> lockControl(String signalId, PhaseLock... locks) {
        return null;
    }

    @Override
    public <T> Result<T> stepControl(String signalId, String sourceType, String signalType, Integer command, Integer stepNum) {
        return null;
    }

    @Override
    public <T> Result<T> allRedControl(String signalId, String sourceType, String signalType, Integer command) {
        Integer runningMode = command.equals(Constants.TRUE) ? RED_RUNNING_MODE : NORMAL_RUNNING_MODE;
        return updateManual(signalId, runningMode);
    }

    @Override
    public <T> Result<T> yellowLightControl(String signalId, String sourceType, String signalType, Integer command) {
        return null;
    }

    @Override
    public <T> Result<T> closeLightControl(String signalId, String sourceType, String signalType, Integer command) {
        return null;
    }

    @Override
    public <T> Result<T> recoverSchedule(String signalId, String sourceType, String signalType, Integer command) {
        return null;
    }

    @Override
    public <T> Result<T> phaseTimingSend() {
        return null;
    }

    @Override
    public <T> Result<T> phaseDiffSend() {
        return null;
    }

    /**
     * @param signalId 信号机编号
     * @param runningMode 0-多时段控制模式 1-关灯、2-黄闪、3-全红、11-取消步进控制、12-灯态锁定、13-灯态解锁
     */
    private <T> Result<T> updateManual(String signalId, Integer runningMode) {
        //执行恢复时间表方案命令
        if (NORMAL_RUNNING_MODE.equals(runningMode) || CANCEL_STEP_MODE.equals(runningMode)) {
            //根据实时灯态获取当前运行模式
            SignalLightStateVo signalInfoVo = getSignalInfoVos(signalId).get(0);
            //如果当前模式已经是定周期，就不在执行恢复时间表方案指令
            if (FIXED_CYCLE_MODE.equals(Integer.valueOf(signalInfoVo.getRunMode()))) {
                throw new ControlException("当前控制模式是定周期控制，请勿下发恢复命令");
            }
        }

        JSONObject bodyObjectParam = new JSONObject();
        bodyObjectParam.put("crossCode", signalId);
        bodyObjectParam.put("controlType", runningMode);
        bodyObjectParam.put("controlNo", 0);
        //控制时长，单位 s，0 则持续控制
        bodyObjectParam.put("duration", 0);
        return setSignalControl(bodyObjectParam);
    }

    private <T> Result<T> setSignalControl(JSONObject bodyObjectParam) {
        try {
            Map<String, String> path = Constants.getPathMapByApiCode("updateLockPhase");
            String strResult = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, path, bodyObjectParam.toJSONString(), null, null, "application/json", null);
            JSONObject object = JSON.parseObject(strResult);
            if (HK_SUCCESS_CODE.equals(object.getInteger(HK_CODE_KEY))) {
                return Result.success();
            }
        } catch (Exception e) {
            log.error("系统繁忙，服务器端内部错误！", e);
        }
        return Result.error("下发指令失败");
    }

    /**
     * 根据信号机获取灯态列表
     */
    private List<SignalLightStateVo> getSignalInfoVos(String signalId) {
        List<BaseCrossInfo> baseSignals = new ArrayList<>();
        BaseCrossInfo baseSignal = new BaseCrossInfo();
        baseSignal.setTelesemeId(signalId);
        baseSignal.setManufacturerAbbr(BasicEnum.ManufacturerEnum.HK.getAbbr());
        baseSignals.add(baseSignal);
        return hkGetSignalMethodService.queryHkSignalInfo(baseSignals);
    }
}
